<?php
/**
 * 
 */
namespace Copilot\Client;

require_once("class.AssetCollection.php");
require_once("class.Video.php");
require_once("class.Event.php");

/**
 * This class contains the base data for a single Artist.
 * The class is suitable for showing an artists detail page however it does not
 * contain any data to events the artist appears on.
 * @author M. Fischboeck
 * @since 15.28
 *
 */
class Artist {
	
	/**
	 * @internal
	 * @var unknown
	 */
	private $id;
	
	/**
	 * @internal
	 * @var unknown
	 */
	private $_data;
	
	/**
	 * @internal
	 * @var unknown
	 */
	private $assets;
	
	/**
	 * Internal constructor. Should not be used by a client.
	 * @internal
	 * @param unknown $data The json data structure from the API
	 */
	public function __construct($data) {
		$this->_data = $data;
		$this->assets = new AssetCollection($this->_data->assets);
	}
	
	/**
	 * Returns the id of the artist
	 * @return string The id of the artist
	 */
	public function getId() {
		return $this->_data->id;
	}
	
	/**
	 * Returns the artists stage name
	 * @return string The Artists official stage name
	 */
	public function getStageName() { 
		return $this->_data->stageName;
	}
	
	/**
	 * Returns the domain to the artists website
	 * @return string URL to the artists website
	 */
	public function getWebsite() {
		return $this->_data->web;
	}
	
	/**
	 * Returns a string where the artists facebook page is hosted
	 * @return string The URL to the artists facebook page
	 */
	public function getFacebook() {
		return $this->_data->facebook;
	}
	
	/**
	 * Returns a string where the artists myspace page is hosted
	 * @return string The URL to the artists myspace page
	 */
	public function getMyspace() {
		return $this->_data->myspace;
	}
	
	/**
	 * Returns the short biography text of the artist.
	 * The string that is being returned is in plaintext and should not
	 * contain any HTML Tags.
	 * @return string The artists short unformatted biography
	 */
	public function getShortBio() {
		return $this->_data->shortBio;
	}
	
	/**
	 * Returns the full biography text of the artist.
	 * The string that is being returned contains HTML tags.
	 * @return string The artists long biography in HTML format
	 */
	public function getLongBio() {
		return $this->_data->longBio;
	}

	public function getImageLink($width) {
		$retval = $this->getMainImageLinks($width);
		if (count($retval) != 0) return $retval[0];
		
		$retval = $this->getBandLogoLinks($width);
		if (count($retval) != 0) return $retval[0];
		
		$retval = $this->getPressImageLinks($width);
		if (count($retval) != 0) return $retval[0];
		return "";
	}
	
	/**
	 * Returns hyperlinks to the artists main images.
	 * @param integer $width The width the images should have
	 * @return array An array containing URLs to the artists main images
	 */
	public function getMainImageLinks($width) {
		$retval = array();
		$tmp = $this->assets->getAssetsByTagname("ARTIST_MAINIMAGE");
		foreach ($tmp as $a) {
			if (strpos($a->getMimeType(), "image/") == 0) 
				array_push($retval, $a->getDownloadUrl($width));
		}
		return $retval;
	}
	
	/**
	 * Returns the assets that are tagged as main image
	 * @return array An array of assets tagged as MAIN_IMAGE
	 */
	public function getMainImages() {
		$retval = array();
		return $this->assets->getAssetsByTagname("ARTIST_MAINIMAGE");
	}
	
	/**
	 * Returns hyperlinks to the artists press images
	 * @param integer $width The width the images should have
	 * @return array An array containing URLs to the artists press images
	 */
	public function getPressImageLinks($width) {
		$retval = array();
		$tmp = $this->assets->getAssetsByTagname("ARTIST_PRESSIMAGE");
		foreach ($tmp as $a) {
			if (strpos($a->getMimeType(), "image/") == 0) 
				array_push($retval, $a->getDownloadUrl($width));
		}	
		return $retval;
	}
	
	/**
	 * Returns an array of assets that are tagged as ARTIST_PRESSIMAGE
	 * @return array An array of assets with the artists press images
	 */
	public function getPressImages() {
		return $this->assets->getAssetsByTagname("ARTIST_PRESSIMAGE");
	}
	
	/**
	 * Returns an array of assets that are tagged as ARTIST_BIOGRAPHY
	 * @return array An array of assets with the artists biography documents
	 */
	public function getBiographyDocuments() {
		return $this->assets->getAssetsByTagname("ARTIST_BIOGRAPHY");
	}
	
	/**
	 * Returns an array of assets that are tagged as ARTIST_PRESSTEXT
	 * @return array An array of assets with the artists presstext documents
	 */
	public function getPressTextDocuments() {
		return $this->assets->getAssetsByTagname("ARTIST_PRESSTEXT");
	}
	
	/**
	 * Returns an array of hyperlinks for assets that are tagged as ARTIST_BANDLOGO
	 * @param integer $width The width the images should have
	 * @return array Array of URLs of the artist bandlogo
	 */
	public function getBandLogoLinks($width) {
		$retval = array();
		$tmp = $this->assets->getAssetsByTagname("ARTIST_BANDLOGO");
		foreach ($tmp as $a) {
			array_push($retval, $a->getDownloadUrl($width));
		}
		return $retval;
	}
	
	/**
	 * Returns an array of assets that are tagged as ARTIST_BANDLOGO
	 * @return array An array of assets containing the artists band logo
	 */
	public function getBandLogos() {
		return $this->assets->getAssetsByTagname("ARTIST_BANDLOGO");
	}

}