<?php
/**
 * 
 */
namespace Copilot\Client;

require_once('class.Artist.php');
require_once('class.Event.php');
require_once('class.Video.php');
require_once('class.News.php');
require_once('class.Page.php');

/**
 * This is the base class for usage by 3rd party developers.
 * This class provides methods to retrieve all public information from a co*pilot installation.
 * This ranges from artists to events (and concerts) to newsfeed and newsletter subscription.
 * 
 * The client is constructed by providing the baseurl to the copilot installation the client
 * should talk with. For information on the base url please contact us at support@copilot-office.de
 * 
 * @author M. Fischboeck
 * @since 15.28
 *
 */
class CopilotClient {
	
	/**
	 * @internal
	 * @var unknown
	 */
	private $baseurl;
	
	/**
	 * Creates a new client for the given base url
	 * @param string $baseurl The URL the client will retrieve it's information from
	 */
	public function __construct($baseurl) {
		$this->baseurl = $baseurl;
		date_default_timezone_set("Europe/Berlin");
	}
	
	/**
	 * Returns all artists that are managed by the agency
	 * @param Page 		$page	The page object to be used
	 * @return Artist[] Array of artists managed by the agency
	 */
	public function getAgencyArtists(Page &$page = NULL) {
		if ($page == NULL)
			$page = new Page(30, 1);
		
		$url = $this->baseurl . '/public/api/artists/search?type=SUEDPOLARTIST&page.sort=stageName,asc&page.page=' . $page->number . '&page.size=' . $page->size;
		return $this->requestArtists($url, $page);
	}
	
	/**
	 * Returns all artists that are not managed by the agency (artists booked for concerts)
	 * @param Page 		$page 	The page object to be used
	 * @return Artist[] Array of artists not managed by the agency
	 */
	public function getExternalArtists(Page &$page = NULL) {
		if ($page == NULL) {
			$page = new Page(30, 1);
		}
		$url = $this->baseurl . '/public/api/artists/search?type=EXTERNAL&page.sort=stageName,asc&page.page=' . $page->number . '&page.size=' . $page->size;
		return $this->requestArtists($url, $page);
	}
	
	/**
	 * Returns a artists details for the given id
	 * @param string 	$id 	The id of the artist
	 * @return Artist 	The artist to be returned
	 */
	public function getArtistById($id) {
		$url = $this->baseurl . '/public/api/artists/' . $id;
		$data = json_decode(file_get_contents($url));
		return new Artist($data);
	}
	
	
	/**
	 * Returns all videos for the given artist
	 * @param Artist 	$artist 	The artist 
	 * @return Video[] 	Array of videos for the given artist. Might be empty but never NULL
	 */
	public function getVideos(Artist $artist) {
		$tmp = file_get_contents($this->baseurl . "/public/api/artists/" . $this->_data->id . "/videoEntries");
		$data = json_decode($tmp);
		$retval = array();
		foreach ($data->content as $vid) {
			array_push($retval, new Video($vid));
		}
		return $retval;
	}
	
	/**
	 * Returns an array of all upcoming events for the given artist
	 * @param Artist 	$artist 	The artist for whom to retrieve future events
	 * @param Page 		$page 		The page object to be used
	 * @return Event[] 	The event for the given artist
	 */
	public function getFutureEvents(Artist $artist, Page &$page = NULL) {
		if ($page == NULL)
			$page = new Page(30, 1);

		$url = $this->baseurl . "/public/api/artists/" . $artist->getId() . "/events/";
		$now = date("Y-m-d");
		$tFu = time() + (24 * 3600 * 365);
		$fut  = date("Y-m-d", $tFu);
		$url .= $now . "/" . $fut;
		$url .= '?page.sort=dateOfEvent,asc&page.page='.$page->number.'&page.size=' . $page->size;

		return $this->requestEvents($url, $page);
	}
	

	/**
	 * Returns all events from the past until the future
	 * @param Page 		$page	The page object to be used
	 * @return Event[] 	All events from the past to the future
	 */	
	public function getAllEvents(Page &$page = NULL) {
		if ($page == NULL)
			$page = new Page(30, 1);
		$url = $this->baseurl . '/public/api/events/?external=false&page.size=' . $page->size . '&page.page=' . $page->number;
		return $this->requestEvents($url, $page);
	}
	
	/**
	 * Returns all events from the given date
	 * @param string 	$from 	The date in format "Y-m-d"
	 * @param Page 		$page	The page object to be used
	 * @return Event[] 	The events in the given timerange
	 */
	public function getEventsFrom($from, Page &$page = NULL) {
		if ($page == NULL)
			$page = new Page(30, 1);
		$url = $this->baseurl . '/public/api/events/from/'.$from.'?external=false&page.size='.$page->size.'&page.page='.$page->number;
		return $this->requestEvents($url, $page);
	}
	
	/**
	 * Returns a single event by it's id
	 * @param string 	$id		The id of the event to be retrieved
	 * @return \Copilot\Client\Event The event
	 */
	public function getEventById($id) {
		$url = $this->baseurl . '/public/api/events/' . $id;
		$data = json_decode(file_get_contents($id));
		return new Event($data);
	}
	
	/**
	 * Returns all concerts from the past to the future (use with care!)
	 * @param Page 		$page	The page object to be used
	 * @return Event[] 	The events
	 */
	public function getAllConcerts(Page &$page = NULL) {
		if (page == NULL)
			$page = new Page(30, 1);
		$url = $this->baseurl . '/public/api/events/?external=true&page.size=' . $page->size . '&page.page=' . $page->number;
		return $this->requestEvents($url, $page);
	}
	
	/**
	 * Returns all concerts from a certain point in time to the future
	 * @param string 	$from 	The date in the format "Y-m-d"
	 * @param Page 		$page	The page object to be used
	 * @return Event[]:
	 */
	public function getConcertsFrom($from, Page &$page) {
		if ($page == NULL)
			$page = new Page(30, 1);
		$url = $this->baseurl . '/public/api/events/from/'.$from.'?external=true&page.size='.$page->size.'&page.page='.$page->number;
		return $this->requestEvents($url);
	}
	
	/**
	 * Returns the concert for the given id
	 * @param string 	$id		The id of the concert to be retrieved
	 * @return \Copilot\Client\Event
	 */
	public function getConcertById($id) {
		return $this->getEventById($id);
	}
	
	
	/**
	 * Returns the news for the given channels or all  news if no channels are specified
	 * @param string 	$channels 	The channels to fetch news from
	 * @param Page 		$page		The page object to be used
	 * @return News[] 	Array of news entries 
	 */
	public function getNews($channels = NULL, Page &$page = NULL) {
		if ($page == NULL)
			$page = new Page(30, 1);
		
		$url = $this->baseurl . '/public/api/news?page.sort=publicationPeriod,desc&page.size=' . $page->size . '&page.page='. $page->number;
		if (isset($channels))
			$url .= '&channel=' . implode(',', $channels);
		$data = json_decode(file_get_contents($url));
		$retval = array();
		foreach ($data->content as $n) {
			array_push($retval, new News($n));
		}
		$this->initPageInfo($data, $page);
		return $retval;
	}
	
	/**
	 * Returns all news for the given artist
	 * @param Artist 	$artist 	The artist to return news for
	 * @param Page 		$page		The page object to be used
	 * @return News[] 	The page of news for the given artist. Might be empty.
	 */
	public function getNewsByArtist(Artist $artist, Page &$page = NULL) {
		if ($page == NULL) 
			$page = new Page(30, 1);
		$url = $this->baseurl . '/public/api/news/?references.type=ARTIST&references.id=' . $artist->getId();
		$url .= '&page.size='.$page->size.'&page.page='.$page->number.'&page.sort=publicationPeriod,desc';
		$data = json_decode(file_get_contents($url));
		$retval = array();
		foreach ($data->content as $n) {
			array_push($retval, new News($n));
		}
		$this->initPageInfo($data, $page);
		return $retval;
	}
	
	/**
	 * Returns the news article for the given id
	 * @param string 	$id 	The id of the news article
	 * @return \Copilot\Client\News The news article
	 */
	public function getNewsById($id) {
		$url = $this->baseurl . '/public/api/news/' . $id;
		$data = json_decode(file_get_contents($url));
		return new News($data);
	}
	
	/**
	 * @internal
	 * @param unknown $url
	 * @return multitype:
	 */
	private function requestArtists($url, Page &$page) {
		$data = json_decode(file_get_contents($url));
		$retval = array();
		foreach ($data->content as $artist) {
			array_push($retval, new Artist($artist));
		}
		
		$this->initPageInfo($data, $page);
		return $retval;
	}
	
	
	/**
	 * @internal
	 * @param unknown $url
	 * @return multitype:
	 */
	private function requestEvents($url, Page &$page) {
		$data = json_decode(file_get_contents($url));
		$retval = array();
		foreach ($data->content as $event) {
			array_push($retval, new Event($event));
		}

		$this->initPageInfo($data, $page);
		return $retval;
	}
	
	/**
	 * @internal
	 * @param unknown $data
	 * @param Page $page
	 */
	private function initPageInfo($data, Page &$page) {
		if ($page == NULL)
			return;
		$page->first = $data->first;
		$page->last  = $data->last;
		$page->direction = $data->sort[0]->direction;
		$page->number = $data->number;
		$page->size   = $data->size;
		$page->sort   = $data->sort[0]->property;
		$page->totalElements = $data->totalElements;
		$page->totalPages    = $data->totalPages;
	}
}